@@ -7,56 +7,58 @@ Usage:
   liquidctl [options] set <channel> speed (<temperature> <percentage>) ...
   liquidctl [options] set <channel> speed <percentage>
   liquidctl [options] set <channel> color <mode> [<color>] ...
+  liquidctl [options] set <channel> screen <mode> [<value>]
   liquidctl --help
   liquidctl --version
 
 Device selection options (see: list -v):
-  -m, --match <substring>        Filter devices by description substring
-  -n, --pick <number>            Pick among many results for a given filter
-  --vendor <id>                  Filter devices by hexadecimal vendor ID
-  --product <id>                 Filter devices by hexadecimal product ID
-  --release <number>             Filter devices by hexadecimal release number
-  --serial <number>              Filter devices by serial number
-  --bus <bus>                    Filter devices by bus
-  --address <address>            Filter devices by address in bus
-  --usb-port <port>              Filter devices by USB port in bus
+  -m, --match <substring>            Filter devices by description substring
+  -n, --pick <number>                Pick among many results for a given filter
+  --vendor <id>                      Filter devices by hexadecimal vendor ID
+  --product <id>                     Filter devices by hexadecimal product ID
+  --release <number>                 Filter devices by hexadecimal release number
+  --serial <number>                  Filter devices by serial number
+  --bus <bus>                        Filter devices by bus
+  --address <address>                Filter devices by address in bus
+  --usb-port <port>                  Filter devices by USB port in bus
 
 Animation options (devices/modes can support zero or more):
-  --speed <value>                Abstract animation speed (device/mode specific)
-  --time-per-color <value>       Time to wait on each color (seconds)
-  --time-off <value>             Time to wait with the LED turned off (seconds)
-  --alert-threshold <number>     Threshold temperature for a visual alert (°C)
-  --alert-color <color>          Color used by the visual high temperature alert
-  --direction <string>           If the pattern should move forward or backward.
-  --start-led <number>           The first led to start the effect at
-  --maximum-leds <number>        The number of LED's the effect should apply to
+  --speed <value>                    Abstract animation speed (device/mode specific)
+  --time-per-color <value>           Time to wait on each color (seconds)
+  --time-off <value>                 Time to wait with the LED turned off (seconds)
+  --alert-threshold <number>         Threshold temperature for a visual alert (°C)
+  --alert-color <color>              Color used by the visual high temperature alert
+  --direction <string>               If the pattern should move forward or backward.
+  --start-led <number>               The first led to start the effect at
+  --maximum-leds <number>            The number of LED's the effect should apply to
 
 Other device options:
-  --single-12v-ocp               Enable single rail +12V OCP
-  --pump-mode <mode>             Set the pump mode (certain Corsair coolers)
-  --temperature-sensor <number>  The temperature sensor number for the Commander Pro
-  --legacy-690lc                 Use Asetek 690LC in legacy mode (old Krakens)
-  --non-volatile                 Store on non-volatile controller memory
-  --direct-access                Directly access the device despite kernel drivers
-  --unsafe <features>            Comma-separated bleeding-edge features to enable
+  --single-12v-ocp                   Enable single rail +12V OCP
+  --pump-mode <mode>                 Set the pump mode (certain Corsair coolers)
+  --fan-mode <channel>:<mode>[,...]  Set the mode for each fan (certain Corsair devices)
+  --temperature-sensor <number>      The temperature sensor number for the Commander Pro
+  --legacy-690lc                     Use Asetek 690LC in legacy mode (old Krakens)
+  --non-volatile                     Store on non-volatile controller memory
+  --direct-access                    Directly access the device despite kernel drivers
+  --unsafe <features>                Comma-separated bleeding-edge features to enable
 
 Other interface options:
-  -v, --verbose                  Output additional information
-  -g, --debug                    Show debug information on stderr
-  --json                         JSON output (list/initialization/status)
-  --version                      Display the version number
-  --help                         Show this message
+  -v, --verbose                      Output additional information
+  -g, --debug                        Show debug information on stderr
+  --json                             JSON output (list/initialization/status)
+  --version                          Display the version number
+  --help                             Show this message
 
 Deprecated:
   -d, --device <index>           Select device by listing index
-  --hid <ignored>                Ignored
 
-Copyright (C) 2018–2022  Jonas Malaco, Marshall Asch, CaseySJ, Tom Frey, Andrew
-Robertson, ParkerMc and contributors
+Copyright 2018–2023 Jonas Malaco, Marshall Asch, CaseySJ, Tom Frey, Andrew
+Robertson, ParkerMc, Aleksa Savic, Shady Nawara and contributors
 
-liquidctl incorporates work by leaty, Ksenija Stanojevic, Alexander Tong, Jens
-Neumaier, Kristóf Jakab, Sean Nelson, Chris Griffith, notaz, realies and Thomas
-Pircher.
+Some modules also incorporate or use as reference work by leaty, Ksenija
+Stanojevic, Alexander Tong, Jens Neumaier, Kristóf Jakab, Sean Nelson, Chris
+Griffith, notaz, realies and Thomas Pircher. This is mentioned in the module
+docstring, along with appropriate additional copyright notices.
 
 SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -69,6 +71,7 @@ import datetime
 import errno
 import inspect
 import json
+import locale
 import logging
 import os
 import platform
@@ -82,8 +85,8 @@ from docopt import docopt
 
 from liquidctl import __version__
 from liquidctl.driver import *
-from liquidctl.error import NotSupportedByDevice, NotSupportedByDriver, UnsafeFeaturesNotEnabled
-from liquidctl.util import color_from_str
+from liquidctl.error import LiquidctlError
+from liquidctl.util import color_from_str, fan_mode_parser
 
 
 # conversion from CLI arg to internal option; as options as forwarded to bused
@@ -116,6 +119,7 @@ _PARSE_ARG = {
     '--legacy-690lc': bool,
     '--non-volatile': bool,
     '--direct-access': bool,
+    '--fan-mode': lambda x: fan_mode_parser(x),
     '--unsafe': lambda x: x.lower().split(','),
     '--verbose': bool,
     '--debug': bool,
@@ -279,6 +283,8 @@ def _device_set_color(dev, args, **opts):
     color = map(color_from_str, args['<color>'])
     dev.set_color(args['<channel>'].lower(), args['<mode>'].lower(), color, **opts)
 
+def _device_set_screen(dev, args, **opts):
+    dev.set_screen(args["<channel>"], args["<mode>"], args["<value>"], **opts)
 
 def _device_set_speed(dev, args, **opts):
     if len(args['<temperature>']) > 0:
@@ -289,9 +295,6 @@ def _device_set_speed(dev, args, **opts):
 
 
 def _make_opts(args):
-    if args['--hid']:
-        _LOGGER.warning('ignoring --hid %s: deprecated option, API will be selected automatically',
-                        args['--hid'])
     opts = {}
     for arg, val in args.items():
         if val is not None and arg in _PARSE_ARG:
@@ -300,8 +303,14 @@ def _make_opts(args):
     return opts
 
 
-def _log_requirements():
-    _LOGGER.debug('python: %s', sys.version)
+def _log_env_infos():
+    _LOGGER.debug('script: %s', sys.argv[0])
+    _LOGGER.debug('version: %s', __version__)
+    _LOGGER.debug('platform: %s', platform.platform())
+    _LOGGER.debug('python: %s', sys.version.replace('\n', ' '))
+    _LOGGER.debug('encoding: %s current, %s preferred, utf8_mode %s',
+                  locale.getlocale()[1], locale.getpreferredencoding(), sys.flags.utf8_mode)
+
     if sys.hexversion >= 0x03080000:
         from importlib.metadata import distribution, version, PackageNotFoundError
 
@@ -314,9 +323,9 @@ def _log_requirements():
         for req in dist.requires:
             name = re.search('^[a-zA-Z0-9]([a-zA-Z0-9._-]*)', req).group(0)
             try:
-                _LOGGER.debug('%s: %s', name, version(name))
+                _LOGGER.debug('with %s: %s', name, version(name))
             except Exception as err:
-                _LOGGER.debug('%s: version n/a (%s)', name, err)
+                _LOGGER.debug('with %s: version n/a (%s)', name, err)
     else:
         _LOGGER.debug('importlib.metadata not available')
 
@@ -390,9 +399,7 @@ def main():
     log_handler.setFormatter(log_fmtter)
     logging.basicConfig(level=log_level, handlers=[log_handler])
 
-    _LOGGER.debug('liquidctl: %s', __version__)
-    _LOGGER.debug('platform: %s', platform.platform())
-    _log_requirements()
+    _log_env_infos()
 
     if __name__ == '__main__':
         _LOGGER.warning('python -m liquidctl.cli is deprecated, prefer python -m liquidctl')
@@ -468,29 +475,26 @@ def main():
                     _device_set_speed(dev, args, **opts)
                 elif args['set'] and args['color']:
                     _device_set_color(dev, args, **opts)
+                elif args['set'] and args['screen']:
+                    _device_set_screen(dev, args, **opts)
                 else:
                     assert False, 'unreachable'
+        except LiquidctlError as err:
+            errors.log(f'{dev.description}: {err}', err=err)
         except OSError as err:
             # each backend API returns a different subtype of OSError (OSError,
             # usb.core.USBError or PermissionError) for permission issues
             if err.errno in [errno.EACCES, errno.EPERM]:
-                errors.log(f'insufficient permissions to access {dev.description}', err=err)
+                errors.log(f'{dev.description}: insufficient permissions', err=err)
             elif err.args == ('open failed', ):
                 errors.log(
-                    f'could not open {dev.description}, possibly due to insufficient permissions',
+                    f'{dev.description}: could not open, possibly due to insufficient permissions',
                     err=err
                 )
             else:
-                errors.log(f'unexpected OS error with {dev.description}', err=err, show_err=True)
-        except NotSupportedByDevice as err:
-            errors.log(f'operation not supported by {dev.description}', err=err)
-        except NotSupportedByDriver as err:
-            errors.log(f'operation not supported by driver for {dev.description}', err=err)
-        except UnsafeFeaturesNotEnabled as err:
-            features = ','.join(err.args)
-            errors.log(f'missing --unsafe features for {dev.description}: {features!r}', err=err)
+                errors.log(f'{dev.description}: unexpected OS error', err=err, show_err=True)
         except Exception as err:
-            errors.log(f'unexpected error with {dev.description}', err=err, show_err=True)
+            errors.log(f'{dev.description}: unexpected error', err=err, show_err=True)
 
     if errors.is_empty() and args['--json']:
         # use __str__ for values that cannot be directly serialized to JSON
