@@ -1,3 +1,9 @@
+"""Access to Linux HWMON data.
+
+Copyright Jonas Malaco and contributors
+SPDX-License-Identifier: GPL-3.0-or-later
+"""
+
 # uses the psf/black style
 
 import logging
