@@ -107,7 +107,7 @@ the devices must be manually initialized by calling `initialize()`.  This will
 cause all connected fans and LED accessories to be detected, and enable status
 updates.  It is recommended to initialize the devices at every boot.
 
-Copyright (C) 2018–2022  Jonas Malaco, CaseySJ and contributors
+Copyright Jonas Malaco, CaseySJ and contributors
 SPDX-License-Identifier: GPL-3.0-or-later
 """
 
@@ -428,9 +428,13 @@ class SmartDevice2(_BaseSmartDevice):
             'speed_channel_count': 3,
             'color_channel_count': 2
         }),
-        (0x1e71, 0x2019, 'NZXT RGB & Fan Controller', {
+        (0x1e71, 0x2011, 'NZXT RGB & Fan Controller (3+6 channels) (experimental)', {
             'speed_channel_count': 3,
-            'color_channel_count': 2
+            'color_channel_count': 0
+        }),
+        (0x1e71, 0x2019, 'NZXT RGB & Fan Controller (3+6 channels) (experimental)', {
+            'speed_channel_count': 3,
+            'color_channel_count': 0
         }),
     ]
 
@@ -660,7 +664,7 @@ class SmartDevice2(_BaseSmartDevice):
 
 class H1V2(SmartDevice2):
     _MATCHES = [
-        (0x1e71, 0x2015, 'NZXT H1 V2 (experimental)', {
+        (0x1e71, 0x2015, 'NZXT H1 V2', {
             'speed_channel_count': 2,
             'color_channel_count': 0
         }),
